//========================================================================================
//
// X-fake.c - A skeleton device control module.
//
// Part of the Adobe Premiere 4.2 Plug-in Developer's Toolkit.
//
// Copyright  1993-96, Adobe Systems Incorporated, all rights reserved worldwide.
//
// Written by Randy Ubillos, Bryan K. "Beaker" Ressler, Nick Schlott.
//
// Version	1.00	10/20/93	Original version.
// Version	1.01	10/30/94	Windows version.
// Version	1.02	1/9/96		Updated for Premiere 4.2 and MSVC++ 2.2 & 4.2.
//
//========================================================================================

//========================================================================================
// Includes
//========================================================================================

#include <windows.h>

#include "Compat.h"
#include "Premiere.h"


//========================================================================================
// Types
//========================================================================================
typedef struct {
	long	pos;
	long	target;
	short	mode;			// The "last reported mode" our device was in.
} LocalRec, **LocalHand;


BOOL WINAPI DllMain (HINSTANCE hDLL, DWORD dwReason, LPVOID lpReserved)
{
	switch (dwReason)
	{
		case DLL_PROCESS_ATTACH:
			break;

		case DLL_THREAD_ATTACH:
			break;

		case DLL_THREAD_DETACH:
			break;

		case DLL_PROCESS_DETACH:
			break;
	}
	return(TRUE);
}


//========================================================================================
// Process commands to the device
//========================================================================================
short PRMEXPORT xDevice (short selector, DeviceHand theData)
{
	short		result = 0, step;
	LocalHand	ldata;

	switch (selector) 
	{
		case dsInit: 
			// Create any local data here and return an error if could not complete. You
			// might put a dialog for parameters here as well. This message is sent when
			// user chooses a new device.
			
			// Allocate our locals handle and stuff it into theData. This will persist
			// across calls.
			(*theData)->deviceData = NewHandleClear(sizeof(LocalRec));
		
			// set up any default values in the local data handle
		
			if (!(*theData)->deviceData) {		// we got a RAM error
			break;
			}
			// fall through...
		
		case dsRestart:
			// This message is the same as dsInit, exept the data handle has already been
			// filled in (from data saved in a project file or the preferences file). You
			// should verify the device connection, and open any needed drivers. This mes-
			// sage is sent when restarting Premiere.
			if ((*theData)->deviceData)
				SetHandleSize((Handle)(*theData)->deviceData,sizeof(LocalRec));
			if ((*theData)->deviceData == nil) 
				break;
			HLock((*theData)->deviceData);
			ldata = (LocalHand)(*theData)->deviceData;
		break;
		
		case dsSetup:									
			// This message is sent when the user clicks the "Options" button in the
			// Device Control preferences dialog. Return a non-zero value if the new set-
			// tings fail (for instance, if they specify a different device and CheckCon-
			// ection fails for that new device, return a non-zero value).
		break;
		
		case dsCleanup:									
			// Here is where we dispose of anything we allocated and close drivers that we
			// opened. This message is sent at quit time.
			DisposHandle((*theData)->deviceData);
			(*theData)->deviceData = nil;
		break;
		
		case dsExecute:									
			// This message is where we perform actual device control and device inquiry
			// tasks. Cache up ldata and then act based on the command.
			ldata = (LocalHand)(*theData)->deviceData;
			switch ((*ldata)->mode) {
				case modePlay:
					(*ldata)->pos++;
					break;
				case modeFastFwd:
					(*ldata)->pos += 10;
					break;
				case modeRewind:
					if ((*ldata)->pos > 10) (*ldata)->pos -= 10;
					else (*ldata)->pos = 0;
					break;
				case modeGoto:
					step = ((*ldata)->target > (*ldata)->pos) ? 1 : -1;
					(*ldata)->pos += step;
					if ((*ldata)->pos == (*ldata)->target) {
						(*ldata)->mode = modePause;
					}
					break;
			}
			switch ((*theData)->command) 
			{
				case cmdGetFeatures:					
					// Here's where we tell Premiere about the features this device has.
					// We can OR as many of these together as apply.
					(*theData)->features = fBasic+fStepFwd+fStepBack+fRecord+
										fPositionInfo+fGoto+fHasOptions+fCanLocate;
				break;
			
				case cmdStatus:								
					// Premiere wants to know the current status of the device we're con-
					// nected to. Often it is not possible to instantly know the status of
					// a device. Premiere does cmdStatus calls all the time, and it
					// behooves the DevC not to waste any time here. The way some of the
					// built-in Premiere modules handle this is by keeping a "last record-
					// ed mode/location" value in their local data, and they use asynch-
					// ronous calls to their drivers. Each time into cmdStatus, if the
					// last async status call you made to your driver is ready, you copy
					// the mode and location into your "last recorded" values. If there's
					// not a status call queued up, go ahead and queue one up to your
					// driver. Finally, at the end of each cmdStatus call, just return
					// your last-recorded mode and location.
				(*theData)->timeformat = 0;				// 0: non-drop, 1: drop-frame
				(*theData)->timerate = 30;				// 24, 25 and 30 are valid
				(*theData)->timecode = (*ldata)->pos;	// Current device location  
				(*theData)->mode = (*ldata)->mode;		// current mode
			break;
		
			case cmdNewMode:								
					// Here Premiere wants you to change the device into a new mode. If we
					// can't do it, the next cmdStatus call will reflect the old mode.
					// Normally you send the commands to the device here, even if they are
					// asynchronous.
					switch ((*theData)->mode) {
						case modeStepFwd:
							(*ldata)->pos++;
							break;
						case modeStepBack:
							(*ldata)->pos--;
							break;
						default:
							(*ldata)->mode = (*theData)->mode;	// change the mode here
							break;
					}
			break;
			
			case cmdGoto:									
					// Premiere is telling you to move the device to the timecode spec-
					// ified in theData. Usually you start the device searching and return
					// without waiting for it to finish. Then Premiere will hammer
					// cmdStatus until the time matches the requested time. When the deck
					// is searching, your cmdStatus should return the mode as "modeGoto",
					// which will result in the text "Searching..." showing up in the
					// timecode display. This is the sorta lame, let-Premiere-do-the-work
					// way of doing timecode location -- if you can do cmdLocate, do it.

					// Do whatever's necessary to move the device to (*theData)->timecode.
					(*ldata)->mode = modeGoto;
					(*ldata)->target = (*theData)->timecode;
			break;
			
			case cmdLocate:								
					// This is the synchronous way of doing timecode location. Premiere
					// wants you to move the deck to the specified timecode and return
					// with the deck in play mode rolling over timecode. You should use
					// the most accurate positioning you can here. Remember that you can
					// send yourself cmdStatus messages to find out what time your over.
					// Often you'll use some "rough seek" command to get your self within
					// a second or two before the requested code, then send yourself a
					// cmdNewMode to go into play mode. Then you sit in a tight loop
					// sending yourself cmdStatus messages until the time is equal to the
					// requested timecode, then you return as fast as possible. Don't try
					// to compensate for the latency between when you return and when
					// video actually gets captured. This is not really within your con-
					// trol, and Premiere has a calibration feature to take care of that
					// for you (by providing a preroll value).
			break;    
			
			case cmdShuttle:
					// Here Premiere is telling you to shuttle the deck either forward
					// or backward at a certain speed (the value is in (*theData)->mode).
					// -100 means 2x backwards playback, -50 means 1x backwards playback,
					// 0 means pause, 50 means 1x forward playback, and 100 means 2x
					// forwards playback. You may have to tweak this mapping to get the
					// right feel.
			break;

			case cmdJogTo:
					// Here Premiere is telling you to quickly locate the deck at
					// (*theData)->timecode. That time will never be far from where you
					// currently are. The value for timecode is calculated by the distance
					// the user dragged the tractor tread on the screen.
			break;
		}
		break;
	}
	return(result);
}
